# coding: utf-8
import threading
import tqdm

class MultiThread():
    """
    多线程任务调用控制

    **Usage:**

    ```
    >>> from sunday.core.MultiThread import MultiThread
    >>> handler = lambda items, threadId, barUpdate: print(item)
    >>> MultiThread([1, 2, 3, 4], lambda *args: [handler, args], 2).start()
    ```

    **Parameters:**

    * **items:** `list` -- 任务列表
    * **func:** `def` -- 根据配置返回target及args, 可返回字典或数组, 如果是数组则第一个值为方法第二个值为入参
    * **tnum:** `int` -- 线程数, 如果传入的items未按线程数分组则可传入tnum程序自动分组，如已分组该值勿传

    **Return:** `thread`
    """
    def __init__(self, items, func, tnum=None):
        if tnum: items = [[item for item in items[i::tnum]] for i in range(tnum)]
        self.threads = []
        self.data_count = 0
        self.pbar = None
        def pbar_update(step=1):
            if self.pbar:
                self.pbar.update(step)
        for idx, item in enumerate(items):
            self.data_count += len(item)
            try:
                params = func(item, idx, pbar_update)
            except Exception as e:
                params = func(item, idx)
            args = params if type(params) == dict else {}
            if type(params) == list:
                args['target'], args['args'] = params
            self.threads.append(threading.Thread(**args))
    
    def start(self, isDelay=True, isBar=False):
        """
        开始执行多线程任务

        **Parameters:**

        * **isDelay:** `bool` -- 是否等待执行完成，默认为True，False表示后台运行
        * **isBar:** `bool` -- 是否显示进度条, 默认False
        """
        for i in self.threads:
            if not isDelay: i.setDaemon(True)
            i.start()
        if isDelay:
            isShowBar = isBar and self.data_count > 0
            if isShowBar:
                self.pbar = tqdm.tqdm(total=self.data_count)
            for i in self.threads: i.join()
            if isShowBar:
                self.pbar.close()
                self.pbar = None



import hashlib

def getMd5(value):
    """
    文本生成md5值

    **Parameters:**

    * **value:** `str` -- 文本字符串
    """
    md5 = hashlib.md5()
    md5.update(value.encode('utf-8'))
    return md5.hexdigest()



import execjs
import os
from sunday.core.paths import sundayCwd

execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis')
sm4 = execjs.compile("""
const sm4 = require('%s/sm4.js')

function encrypt(datastrList, publicKey) {
    return datastrList.map(each => sm4.encrypt(each, publicKey))
}

function decrypt(datastrList, publicKey) {
    return datastrList.map(each => sm4.decrypt(each, publicKey))
}
""" % (execPath, ))

def cryptBySm4Js(datastrList, key, cryptType='encrypt'):
    if not datastrList: return []
    if type(datastrList) == str:
        datastrList = [datastrList]
    return sm4.call(cryptType, datastrList, key)

if __name__ == "__main__":
    publicKey = '596b71524573654d366b627244346b65'
    origin = '{"a":"haha"}'
    target = cryptBySm4Js(origin, publicKey, 'encrypt')
    print(f"========》cryptBySm4('{origin}', '{publicKey}', 'decrypt'): {target}")
    print(f"========》cryptBySm4('{target}', '{publicKey}', 'decrypt'): {cryptBySm4Js(target, publicKey, 'decrypt')}")



import os
from sunday.core.globalvar import getvar, setvar
from sunday.core.globalKeyMaps import sdvar_logger, sdvar_exception
from sunday.core.paths import sundayCwd
from sunday.core.cmdexec import cmdexec

def grenKey(num1=None, num2=None):
    """
    生成指定位数标识码, 如grenKey(16, 61)则生成16位字符码来源前61的随机字符串

    标识码字符来源：0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz

    **Parameters:**

    * **num1:** `int` -- 标识码位数
    * **num2:** `int` -- 标识码标识范围
    """
    import random
    words = list("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz")
    num2 = num2 or len(words)
    ans = [None] * (num1 if num1 is not None else 36)
    a = None
    if num1:
        for n in range(num1):
            ans[n] = words[int(random.random() * num2)]
    else:
        ans[8] = ans[13] = ans[18] = ans[23] = '-'
        ans[14] = '4'
        for n in range(36):
            if not ans[n]: a = int(16 * random.random())
            ans[n] = words[3 & a | 8 if n == 19 else a]
    return ''.join(ans)

def str2base16(oristr):
    """
    字符串转化为十六进制数字串

    **Usage:**
    ```
    >>> from sunday.utils.cryptanalyst import str2base16
    >>> print(str2base16('helloworld'))
    68656c6c6f776f726c64
    ```

    **Parameters:**

    * **oristr:** `str` -- 字符串
    """
    # 字符串转16进制
    return ''.join([hex(ord(s)).replace('0x', '') for s in list(oristr)])

def cryptBySm2(datastrList, key, cryptType='encrypt'):
    """
    国密SM2加解密(非对称加密)

    **Parameters:**

    * **datastrList:** `list` -- 明文或者密闻组成的数组，如一次加密多个密码
    * **key:** `str` -- 加密传公钥, 解密传私钥
    * **cryptType:** `str` -- encrypt(加密)、decrypt(解密)
    """
    if not datastrList: return []
    if type(datastrList) == str:
        datastrList = [datastrList]
    execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis', f'{cryptType}_sm2.js')
    if not os.path.exists(execPath):
        raise getvar(sdvar_exception)(-1, f'{cryptType}暂不支持，解析文件不存在：{execPath}')
    cmd = 'node {execPath} {key} {cipherMode} {datastr}'.format(
            execPath=execPath,
            key=key,
            cipherMode='1',
            datastr=' '.join(datastrList))
    execcode, stdout, stderr = cmdexec(cmd)
    if execcode != 0:
        getvar(sdvar_logger).error(stderr)
        raise getvar(sdvar_exception)(-1, f'sm2 {cryptType} fail')
    return stdout.strip().split('\n')

def cryptBySm4(datastrList, key, cryptType='encrypt'):
    """
    国密SM4加解密(对称加密)

    **Parameters:**

    * **datastrList:** `list` -- 明文或者密闻组成的数组，如一次加密多个密码
    * **key:** `str` -- 公钥
    * **cryptType:** `str` -- encrypt(加密)、decrypt(解密)
    """
    if not datastrList: return []
    if type(datastrList) == str:
        datastrList = [datastrList]
    execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis', f'{cryptType}_sm4.js')
    if not os.path.exists(execPath):
        raise getvar(sdvar_exception)(-1, f'{cryptType}暂不支持，解析文件不存在：{execPath}')
    cmd = "node {execPath} {key} {datastr}".format(
            execPath=execPath,
            key=key,
            datastr=' '.join(datastrList))
    execcode, stdout, stderr = cmdexec(cmd)
    if execcode != 0:
        getvar(sdvar_logger).error(stderr)
        raise getvar(sdvar_exception)(-1, f'sm4 {cryptType} fail')
    return stdout.strip().split('\n')

def cryptByJsEncrypt(datastrList, key, cryptType='encrypt', isHex=False):
    """
    jsencrypt加解密

    **Parameters:**

    * **datastrList:** `list` -- 明文或者密闻组成的数组，如一次加密多个密码
    * **key:** `str` -- 公钥
    * **cryptType:** `str` -- encrypt(加密)、decrypt(解密)
    """
    if not datastrList: return []
    if type(datastrList) == str:
        datastrList = [datastrList]
    execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis', f'{cryptType}_jsencrypt{"_hex" if isHex else ""}.js')
    if not os.path.exists(execPath):
        raise getvar(sdvar_exception)(-1, f'{cryptType}暂不支持，解析文件不存在：{execPath}')
    cmd = 'node {execPath} {key} {datastr}'.format(
            execPath=execPath,
            key=key,
            datastr=' '.join(datastrList))
    execcode, stdout, stderr = cmdexec(cmd)
    if execcode != 0:
        getvar(sdvar_logger).error(stderr)
        raise getvar(sdvar_exception)(-1, f'jsencrypt {cryptType} fail')
    return stdout.strip().split('\n')

def cryptByCryptoJS(datastrList, key, cryptType='encrypt', mode='ECB'):
    """
    cryptoJS加解密

    **Parameters:**

    * **datastrList:** `list` -- 明文或者密闻组成的数组，如一次加密多个密码
    * **key:** `str` -- 公钥
    * **cryptType:** `str` -- encrypt(加密)、decrypt(解密)
    * **mode:** `str` -- 加密方式，默认ECB
    """
    if not datastrList: return []
    if type(datastrList) == str: datastrList = [datastrList]
    execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis', f'{cryptType}_cryptojs_{mode}.js')
    if not os.path.exists(execPath):
        raise getvar(sdvar_exception)(-1, f'{cryptType}暂不支持，解析文件不存在：{execPath}')
    cmd = 'node {execPath} {key} {datastr}'.format(
            execPath=execPath,
            key=key,
            datastr=' '.join(datastrList))
    execcode, stdout, stderr = cmdexec(cmd)
    if execcode != 0:
        getvar(sdvar_logger).error(stderr)
        raise getvar(sdvar_exception)(-1, f'cryptojs {cryptType} {mode} fail')
    return stdout.strip().split('\n')

def cryptByUuid(datastrList, key, cryptType='decrypt'):
    """
    根据密文与密钥解码出UUID值

    **Parameters:**

    * **datastrList:** `list` -- 明文或者密闻组成的数组，如一次加密多个密码
    * **key:** `str` -- 密钥
    * **cryptType:** `str` -- encrypt(加密)、decrypt(解密)
    """
    if not datastrList: return []
    if type(datastrList) == str:
        datastrList = [datastrList]
    execPath = os.path.join(sundayCwd, 'utils', 'cryptanalysis', f'{cryptType}_aestoolsnsrd.js')
    if not os.path.exists(execPath):
        raise getvar(sdvar_exception)(-1, f'{cryptType}暂不支持，解析文件不存在：{execPath}')
    cmd = "node {execPath} {key} {datastr}".format(
            execPath=execPath,
            key=key,
            datastr=' '.join(datastrList))
    execcode, stdout, stderr = cmdexec(cmd)
    if execcode != 0:
        getvar(sdvar_logger).error(stderr)
        raise getvar(sdvar_exception)(-1, f'aesToolsNsrd decrypt UUID {cryptType} fail')
    return stdout.strip().split('\n')

if __name__ == "__main__":
    key = grenKey(16, 61)
    print(f'========》grenKey(16, 61): {key}')
    print(f'========》str2base16("{key}"): {str2base16(key)}')
    publicKey = '596b71524573654d366b627244346b65'
    origin = '{"a":"haha"}'
    target = cryptBySm4(origin, publicKey, 'encrypt')
    print(f"========》cryptBySm4('{origin}', '{publicKey}', 'decrypt'): {target}")
    print(f"========》cryptBySm4('{target}', '{publicKey}', 'decrypt'): {cryptBySm4(target, publicKey, 'decrypt')}")
    # print(f"cryptBySm2({key}, 'howduudu'): {cryptBySm2(key, 'howduudu')}")
    # print(f"cryptByJsEncrypt({key}, 'howduudu', 'encrypt'): {cryptBySm4(key, 'howduudu', 'encrypt')}")



# coding: utf-8
import os
import sunday.core.paths as paths
from configparser import ConfigParser

__all__ = ['getConfig']

defaultConfig = {
    'LOGGING.level': 'ERROR',
    'LOGGING.print_file': False,
    'LOGIN.cookieFile': '.cookies',
    'LOGIN.envFile': '.env',
    # 密钥
    'CRYPTO.key': 'HOWDUUDU'
}

def getConfig(name, file='config.ini', default={}):
    """
    返回能够获取全局配置文件(`$SUNDAY_ROOT/config.ini`)内配置值的函数

    **Parameters:**

    * **name:** `str` -- 键名
    * **file:** `str` -- 文件名
    * **default:** `dict` -- 默认配置的集合，优先级大于全局的配置

    **Return:** `getval(key)`
    """
    configPwd = os.path.join(paths.rootCwd, file)
    cfg = ConfigParser()
    cfg.read(configPwd, encoding='utf-8')
    def getval(key):
        if cfg.has_option(name, key):
            ans = cfg.get(name, key)
            if ans in ['True', 'False']: ans = eval(ans)
            return ans;
        return default.get(key) or defaultConfig.get('.'.join([name, key]))
    return getval

if __name__ == "__main__":
    ans = getConfig('LOGGING')('print_file')
    print(ans)
    print(type(ans))



# coding: utf-8

sdvar_logger = 'sunday_logger'
sdvar_loggerid = 'loggerId'
sdvar_premust = 'sunday_premust'
sdvar_exectime = 'sunday_exectime'
sdvar_exception = 'sunday_exception'
sdvar_loglevel = 'sunday_loglevel'



# coding: utf-8
import sunday.core.globalvar as globalvar

def cache_name(name):
    '''根据名称缓存第一次执行结果'''
    def cache_name_func(func):
        def wrap(*args, **kwargs):
            if globalvar.getvar(name) is None:
                globalvar.setvar(name, func(*args, **kwargs))
            return globalvar.getvar(name)
        return wrap
    return cache_name_func



# coding: utf-8
from prettytable import PrettyTable

def printTable(titleArr=[], align=None, isPrint=True):
    """
    命令工具通常需要输出执行结果，而数据一般以表格的方式输出更符合查看需求，该方法用于生成并打印表格

    **Usage:**

    ```
    >>> from sunday.core.printTable import printTable
    >>> printTable(['姓名', '年龄', '性别'])([['张三', 18, '男'], ['翠花', 20, '女']])
    +------+------+------+
    | 姓名 | 年龄 | 性别 |
    +------+------+------+
    | 张三 |  18  |  男  |
    | 翠花 |  20  |  女  |
    +------+------+------+
    ```

    **Parameters:**

    * **titleArr:** `list` -- 表格头数组
    * **align:** `str` -- 数据对齐方式, l（左对齐）、r（右对齐）、c（局中）
    * **isPrint:** `bool` -- 是否在命令行中打印表格
    """
    table = PrettyTable(titleArr)
    if align:
        for title in titleArr:
            table.align[title] = align
    def log(rowArr=[], alone=False):
        if alone: rowArr = [rowArr]
        for row in rowArr:
            table.add_row(row)
        if isPrint: print(table)
        return table
    return log



# coding: utf-8
import os
import requests
import sunday.core.paths as paths
import sunday.core.globalvar as globalvar
from http.cookiejar import LWPCookieJar, CookieJar
from sunday.core.getConfig import getConfig
from sunday.core.logger import Logger
from sunday.core.fetch import Fetch
from sunday.core.getException import getException
from sunday.core.globalvar import getvar, setvar
from sunday.core.globalKeyMaps import sdvar_exception
from sunday.utils.tools import mergeObj
from pydash import get

LoginError = getException()

class LoginBase():
    """
    LoginBase为登录插件开发必须的原始类，供登录插件继承

    **Usage:**

    ```python
    from sunday.utils.LoginBase import LoginBase
    class MyLogin(LoginBase):
        def __init__(self, useLoginState=True, ident=None):
            self.logger = Logger('MyLogin').getLogger()
            LoginBase.__init__(self, logger=self.logger, ident=ident or '', error=[99999, '网站接口异常，请稍后重试'])
            checkUrl = 'http://host:port/getCurrentUser'
            self.rs, self.isLogin = self.initRs(checkUrl, useLoginState)
    ```

    **Parameters:**

    * **logger:** `Logger` -- 子类调用传入，可以保持日志打印一致
    * **pacWifi:** `str` -- wifi名称，如果当前网络连入的wifi相同则使用pac代理
    * **pacUrl:** `str` -- url地址，尝试请求该url，如果能请求到则使用pac代理，一般为内网链接
    * **ident:** `str` -- 多用户标识，用于多用户认证
    * **error:** `list` -- 数组元素为两个时生效，与`sunday.core.fetch.Fetch.setJsonError`入参值一致
    * **file(已弃用):** `str` -- 目标登录对象执行文件
    """
    def __init__(self, file=paths.cacheCwd, logger=Logger('LoginBase').getLogger(), pacWifi=None, pacUrl=None, ident='', error=[]):
        pwd = os.path.abspath(file)
        if os.path.isdir(pwd) == False: pwd = os.path.dirname(pwd)
        self.logger = logger
        cfg = getConfig('LOGIN')
        ident = ('-' if ident else '') + ident
        self.cookiePwd = os.path.join(pwd, cfg('cookieFile') + ident)
        self.envPwd = os.path.join(pwd, cfg('envFile') + ident)
        self.fetch = Fetch(pacWifi=pacWifi, pacUrl=pacUrl)
        if len(error) == 2: self.fetch.setJsonError(*error)

    def getCookiePwd(self):
        return self.cookiePwd

    def getEnvPwd(self):
        return self.envPwd

    def saveCookie(self):
        '''保存cookie'''
        savefun = get(self.fetch, 'session.cookies.save')
        if callable(savefun): savefun(ignore_discard=True, ignore_expires=True)

    def getCookie(self, useHistory=True):
        """ params(useHistory): 控制是否使用历史登陆态
            return(cookies, boolean): 返回cookie对象和是否登录标志
        """
        filename = self.getCookiePwd()
        hasCookieFile = os.path.isfile(filename)
        if useHistory and hasCookieFile:
            load_cookiejar = LWPCookieJar()
            load_cookiejar.load(filename, ignore_discard=True, ignore_expires=True)
            load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
            self.logger.debug('是否存在登录态: 是')
            return requests.utils.cookiejar_from_dict(load_cookies), True
        if hasCookieFile: os.remove(filename)
        self.logger.debug('是否存在登录态: 否')
        return LWPCookieJar(filename=filename), False

    def setCookie(self, name, value, domain, rest={}, **argvs):
        '''增加新的cookie, 入参参考cookielib.Cookie中的定义'''
        cookies = self.fetch.session.cookies
        standard = mergeObj({ 'path': '/', 'domain': domain, 'version': 0 }, argvs)
        cookie = CookieJar()._cookie_from_cookie_tuple((name, value, standard, rest), None)
        cookies.set_cookie(cookie)
    
    def checkLogin(self, checkUrl=None):
        """
        程序默认的登录成功判断方法, 即: 请求首页, 如果跳到登录页则说明登录态失效

        这种判断方式根据目标网站处理逻辑不同可能会不准确，一般子类会重构这个方法

        **Parameters:**

        * **checkUrl:** `str` -- 用于检查是否登录成功的网站链接
        """
        if checkUrl is None:
            raise getvar(sdvar_exception)(-1, '检查登录状态的链接不能为空')
        self.fetch.get(checkUrl)
        res = self.fetch.get(checkUrl)
        return res.status_code != 200 or res.url != checkUrl

    def initRs(self, checkUrl=None, useHistory=True):
        """
        初始化会话

        **Parameters:**

        * **checkUrl:** `str` -- 用于检查是否登录成功的网站链接
        * **useHistory:** `bool` -- 标记是否使用上一次的登录态，为False则为重新登录不用历史登录态
        """
        session = self.fetch.session
        session.cookies, isLogin = self.getCookie(useHistory=useHistory)
        if isLogin and self.checkLogin(checkUrl):
            session.cookies, isLogin = self.getCookie(useHistory=False)
        return self.fetch, isLogin




# coding: utf-8
import os
from sunday.core.cmdexec import cmdexec
from sunday.core.common import exit
from sunday.core.globalvar import getvar
from sunday.core.globalKeyMaps import sdvar_logger

root_key = 'SUNDAY_ROOT'
path_key = 'PATH'
sunday_root = os.environ.get(root_key) or os.path.join(os.environ['HOME'], '.sunday')
sunday_bin = os.path.join(sunday_root, 'bin')
(sign, *_) = cmdexec('which sunday_install')
if sign != 0:
    cur = os.path.abspath(__file__)
    while not os.path.exists(os.path.join(cur, 'bin/sunday_install')) and cur != '/':
        cur = os.path.dirname(cur)
    if cur != '/':
        sunday_bin += ':%s' % os.path.join(cur, 'bin')
    else:
        exit('sunday已安装，但未找到sunday_install执行目录，请检查！')
exportList = ['# sunday']
if not os.environ.get(root_key):
    exportList.append('export %s=%s' % (root_key, sunday_root))
if os.environ.get(path_key).find(sunday_bin) == -1:
    exportList.append('export %s=%s:\$PATH' % (path_key, sunday_bin))

def getShellRcPath():
    shell = os.path.basename(os.environ.get('SHELL') or '')
    p = ''
    if shell == 'zsh':
        p = '.zshrc'
    elif shell == 'bash':
        p = '.bashrc'
    if p: return os.path.join(os.environ['HOME'], p)
    return False

rctext = '\n'.join(exportList)
rcfile = getShellRcPath()
if rcfile and len(exportList) > 1:
    if not os.path.exists(rcfile): cmdexec('touch %s' % rcfile)
    cmdexec('echo "\n%s" >> %s' % (rctext, rcfile))
    getvar(sdvar_logger).info('sunday 初始化成功！')
else:
    print(rctext)



# coding: utf-8
from . import premust

# 工具函数
from .checkPacNet import *
from .getConfig import *
from .getEnv import *
from .enver import *
from .printTable import *
from .getParser import *
from .common import *
from .cachevar import *
from .cmdexec import *
from .MultiThread import *
from .auth import *
from . import globalvar
from .aesCbc import *
from .getException import *

# 日志
from .logger import *

# 请求
from .fetch import *

# 目录
from . import paths

# 常量键
from .globalKeyMaps import *



# coding: utf-8
import logging
import colorlog
import atexit
import datetime
from os import path
from sunday.core.globalvar import getvar, setvar
from sunday.core.getConfig import getConfig
import sunday.core.paths as paths
from sunday.core.globalKeyMaps import sdvar_loggerid, sdvar_logger, sdvar_loglevel

if not getvar(sdvar_loggerid):
    loggerId = datetime.datetime.today().isoformat()
    setvar(sdvar_loggerid, loggerId)
    if getConfig('LOGGING')('print_file'):
        @atexit.register
        def exitPrintLogfile():
            getvar(sdvar_logger).info('LOG FILE AT: %s' % path.join(paths.logCwd, loggerId))

logLevelKeys = ['debug', 'info', 'warning', 'error', 'critical']

logfile = path.join(paths.logCwd, getvar(sdvar_loggerid))

logging.basicConfig(filename=logfile, level=logging.DEBUG, force=True,
        format='[%(asctime)s.%(msecs)-3d] %(levelname)s <%(name)s>: %(message)s')

class Logger():
    """
    日志管理与控制，日志分类、打印与缓存

    **Usage:**

    ```
    >>> from sunday.core.logger import Logger
    >>> logger = Logger('自定义名称').getLogger()
    >>> logger.debug('日志打印输出')
    ```

    **Parameters:**

    * **name:** `str` -- 日志分类名称
    * **level:** `str` -- 日志打印级别，DEBUG、INFO、WARNING、ERROR、CRITICAL
    * **format:** `str` -- 日志打印格式

    **Return:** `logger`
    """
    def __init__(self, name, level='', format=''):
        cfg = getConfig('LOGGING')
        logger = logging.getLogger(name)
        if not logger.handlers:
            loglevel = level or getvar(sdvar_loglevel) or cfg('level') or 'error'
            logger.setLevel(logging.getLevelName(loglevel.upper()))
            loghd = logging.StreamHandler()
            loghd.setFormatter(colorlog.ColoredFormatter(format or cfg('format'), datefmt='%H:%M:%S'))
            logger.addHandler(loghd)
            logger.propagate = True
        self.logger = logger

    def getLogger(self):
        return self.logger

def setLogLevel(level):
    """
    修改所有已经存在的日志实例等级

    **Parameters:**

    * **level:** `str` -- 日志打印级别，DEBUG、INFO、WARNING、ERROR、CRITICAL
    """
    # 设置所有日志管理者的日志等级
    setvar(sdvar_loglevel, level)
    levelstr = level.upper()
    level = getattr(logging, levelstr)
    handlerList = logging.Logger.manager.loggerDict.values()
    for handler in handlerList:
        if hasattr(handler, 'setLevel') and handler.level != level:
            handler.setLevel(level)



import io
import random
from PIL import Image

def randomColor():
    colorArr = ['1','2','3','4','5','6','7','8','9','A','B','C','D','E','F']
    color = "#"
    for i in range(6): color += colorArr[random.randint(0,14)]
    return color

def grenImageBase(width, height=None, color=None, format='PNG', mode='RGBA'):
    if height is None: height = width
    if color is None: color = randomColor()
    img = Image.new(mode, (width, height), color)
    img_byte = io.BytesIO()
    img.save(img_byte, format=format)
    return img_byte.getvalue()

def grenImagePng(*args, **kwargs):
    """
    生成指定宽高及颜色的png图片

    **Parameters:**

    * **width:** `int` -- 图片宽
    * **height:** `int` -- 图片高
    * **color:** `str` -- 图片颜色，不传时为随机颜色
    """
    return grenImageBase(*args, format='PNG', mode='RGBA', **kwargs)

def grenImageJpeg(*args, **kwargs):
    """
    生成指定宽高及颜色的jpeg图片

    **Parameters:**

    * **width:** `int` -- 图片宽
    * **height:** `int` -- 图片高
    * **color:** `str` -- 图片颜色，不传时为随机颜色
    """
    return grenImageBase(*args, format='JPEG', mode='RGB', **kwargs)

def grenImageJpg(*args, **kwargs):
    """
    生成指定宽高及颜色的jpg图片, 同`grenImageJpeg`

    **Parameters:**

    * **width:** `int` -- 图片宽
    * **height:** `int` -- 图片高
    * **color:** `str` -- 图片颜色，不传时为随机颜色
    """
    return grenImageBase(*args, format='JPEG', mode='RGB', **kwargs)

def grenImageGif(*args, **kwargs):
    """
    生成指定宽高及颜色的gif图片

    **Parameters:**

    * **width:** `int` -- 图片宽
    * **height:** `int` -- 图片高
    * **color:** `str` -- 图片颜色，不传时为随机颜色
    """
    return grenImageBase(*args, format='GIF', mode='RGB', **kwargs)



from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.header import Header
from smtplib import SMTP_SSL
from sunday.core.logger import Logger

logger = Logger('MAIL').getLogger()

class Email(object):
    """
    发送邮件，用于程序运行监控时的异常预警

    **Usage:**

    ```
    >>> from sunday.utils.Email import Email, getEmailFile
    >>> config = {'host': 'smtp.163.com', 'user': '发送方邮箱地址', 'pwd': '发送方邮箱密码', 'info': '发件人描述', 'tars': '收件人邮箱', 'title': '报错预警邮件'}
    >>> email = Email(**config)
    >>> email.send_mail('<center><h3>HTML文本</h3></center>', msg_type='html')
    >>> email.send_mail('纯文本', msg_type='plain')
    >>> email.send_mail('带附件邮件', docs=getEmailFile(b'二进制', '附件.excel'))
    ```

    **Parameters:**

    * **host:** `str` -- smtp服务地址
    * **user:** `str` -- 邮箱账号
    * **pwd:** `str` -- 授权码
    * **tars:** `str` -- 收件人
    * **title:** `str` -- 邮件标题
    * **info:** `str` -- 发件人描述

    **Return:** `email`
    """
    def __init__(self, host=None, user=None, pwd=None, tars=None, info='info', title='title', **argvs):
        self.host = host
        self.user = user
        self.pwd = pwd
        self.tars = tars
        self.info = info
        self.title = title

    def new_msg(self, content, users, msg_type='plain', info=None, title=None, docs=[]):
        msg = MIMEMultipart()
        msg.attach(MIMEText(content, msg_type, 'utf-8'))
        msg['From'] = f'{info or self.info} <{self.user}>'
        msg['To'] = ','.join(users)
        msg['Subject'] = Header(title or self.title, 'utf-8')
        if type(docs) != list: docs = [docs]
        for doc in docs: msg.attach(doc)
        return msg

    def send(self, msg, users):
        server = SMTP_SSL(self.host, 465)
        server.login(self.user, self.pwd)
        server.sendmail(self.user, users, msg.as_string())
        server.quit()

    def send_mail(self, content, users=None, **argvs):
        """
        执行发送邮件

        **Parameters:**

        * **content:** `str` -- 邮件内容
        * **users:** `str` -- 收件人
        * **msg_type:** `str` -- 邮件内容类型，文本(plain)/网页(html)，默认为plain
        * **title:** `str` -- 邮件标题
        * **info:** `str` -- 发件人描述
        * **docs:** `MIMEApplication` -- 附件
        """
        logger.debug('执行邮件发送')
        users = users or self.tars
        if not users: logger.error('收件人必传')
        if type(users) == str: users = [users]
        try:
            msg = self.new_msg(content, users, **argvs)
            self.send(msg, users)
            logger.info('邮件发送成功')
        except Exception as e:
            logger.exception(e)

def getEmailFile(stream, filename):
    """
    生成文件流对象，用于带附件邮件发送使用

    **Parameters:**

    * **stream:** `binary` -- 二进制流文本
    * **filename:** `str` -- 文件名
    """
    file = MIMEApplication(stream)
    file.add_header('Content-Disposition', 'attachment', filename=filename)
    return file

if __name__ == '__main__':
    email = Email('smtp.xxx.com', 'user@xxx.com', 'xxxxxx')
    email.send_mail('content', ['user1@xxx.com', 'user2@xxx.com'], title='title', info='info')



# coding: utf-8
import requests
import time
import sunday.core.paths as paths
import validators
from http.cookiejar import LWPCookieJar, CookieJar
from sunday.core.getEnv import getEnv
from sunday.core.logger import Logger
from sunday.core.checkPacNet import checkPacNet
from sunday.core.getConfig import getConfig
from sunday.core.enver import enver
from sunday.core.auth import Auth, getCurrentUser
from pypac import PACSession
from requests.auth import HTTPProxyAuth
from sunday.core.globalvar import getvar, setvar
from sunday.core.globalKeyMaps import sdvar_exception
from pydash import omit, get

# 关闭由verify=False引起的提示
requests.packages.urllib3.disable_warnings()

headers = {
    'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) '
        'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36'),
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Accept-Encoding': 'gzip, deflate',
}

headers_post = {
    'contentType': {
        'Content-Type': 'application/json;charset=UTF-8'
    }
}

logger = Logger('Fetch').getLogger()
class Fetch():
    """
    对requests进行封装，实现内网环境使用系统pac代理、保持会话、请求失败再次尝试、请求代理、返回结果安全json化等

    **Usage:**

    ```
    >>> from sunday.core.fetch import Fetch
    >>> fetch = Fetch()
    >>> fetch.get('https://www.baidu.com')
    <Response [200]>
    ```

    **Parameters:**

    * **pacWifi:** `str` -- wifi名称，如果当前网络连入的wifi相同则使用pac代理
    * **pacUrl:** `str` -- url地址，尝试请求该url，如果能请求到则使用pac代理，一般为内网链接
    * **pacWifiName(弃用):** `str` -- 与pacWifi一致，为了兼容老代码
    * **proxy:** `str` -- 让请求使用代理，传入如：`127.0.0.1:8888`或者`https://127.0.0.1:5555/getProxy`

    **Return:** `fetch`
    """
    def __init__(self, pacWifi=None, pacUrl=None, pacWifiName=None, proxy=None):
        self.pacWifi = pacWifi or pacWifiName
        self.pacUrl = pacUrl
        self._hasProxy = False
        self._proxystr = proxy
        self._hasPacProxy = False
        self.session = self.get_session(proxy)
        self.jsonErrorMessage = None
        self.jsonErrorNumber = 99999

    def setJsonError(self, jsonErrorMessage='服务器异常请稍后重试!', jsonErrorNumber=99999):
        """
        设置get_json与post_json方法中执行json化失败的报错配置，若配置后返回非json数据则直接报错

        **Usage:**

        ```
        >>> from sunday.core.fetch import Fetch
        >>> fetch = Fetch()

        >>> fetch.get('https://www.baidu.com')
        {'sunday_error': True,
         'msg': '返回结果非JSON',
         'url': 'https://www.baidu.com',
         'text': ''}

        >>> fetch.setJsonError()
        >>> fetch.get('https://www.baidu.com')
        CustomError: '99999 服务器异常请稍后重试!'
        ```

        **Parameters:**

        * **jsonErrorMessage:** `str` -- 报错提示信息, 默认`服务器异常请稍后重试!`
        * **jsonErrorNumber:** `int` -- 报错code编码, 默认`99999`
        """
        self.jsonErrorNumber = jsonErrorNumber
        self.jsonErrorMessage = jsonErrorMessage

    def get_session(self, proxy=None):
        # 返回session
        proxy = proxy or getConfig('PROXY')('proxy')
        session = requests.session() if proxy else PACSession()
        self.openProxy(session, proxy)
        session.headers.update(headers)
        session.verify = False
        return session
    
    def openProxy(self, session, proxy):
        '''开启代理'''
        if proxy:
            self._proxystr = proxy
            if validators.url(proxy) == True:
                proxy = requests.get(proxy).text
            prevProxy = get(session, 'proxies.http')
            if prevProxy is None:
                logger.warning(f'设置代理：{proxy}')
            elif prevProxy != proxy:
                logger.warning(f'更换代理：{prevProxy} => {proxy}')
            session.proxies = {
                'http': proxy,
                'https': proxy
            }
            self._hasProxy = True
        else:
            self.openPacProxy(session)
    
    def add_header(self, headers):
        """
        设置全局请求头

        **Parameters:**

        * **headers:** `dict` -- 添加到请求头的字典
        """
        if type(headers) == dict:
            self.session.headers.update(headers)
    
    def closeProxy(self):
        '''关闭代理'''
        self.session.proxies = None
        self._hasProxy = False
    
    def hasProxy(self):
        '''返回是否开启代理'''
        if self._hasProxy != True: return False
        proxy = get(self.session, 'proxies.http'),
        return {
                'hasProxy': self._hasProxy,
                'proxystr': self._proxystr,
                'proxy': proxy,
                'proxytype': None if proxy == self._proxystr else 'url'
                }

    def hasPacProxy(self):
        '''返回是否开启pac'''
        return self._hasPacProxy

    def openPacProxy(self, session):
        '''检测为内网环境则增加代理认证'''
        if checkPacNet(self.pacWifi, self.pacUrl):
            auth = Auth(paths.envCwd, '[PAC PROXY]')
            getenv = enver(paths.envCwd)[0]
            username = auth.addParams('pac_user', 'PAC代理账号', value=getenv('pac_user'), defaultValue=getCurrentUser())
            password = auth.addParams('pac_pass', 'PAC代理密码', value=getenv('pac_pass'), isPass=True)
            logger.warning('pac认证网络, 网络交互使用代理中')
            session.proxy_auth = HTTPProxyAuth(username, password)
            self._hasPacProxy = True
            return True
        return False

    def requestByType(self, type, times=0, *args, **kwargs):
        try:
            stime = time.time()
            params = omit(kwargs, ['timeout_time'])
            logger.debug('fetching %s %s' % (args, params))
            if 'timeout' not in params: params['timeout'] = 15
            res = getattr(self.session, type)(*args, **params)
            logger.info('fetch result %s (状态: %d, 用时: %.3f)' % (res.url, res.status_code, time.time() - stime))
            return res
        except Exception as e:
            if times >= kwargs.get('timeout_time', 3):
                raise getvar(sdvar_exception)(-1, '第%d次尝试失败, 请检查网络后重试!' % times)
            logger.warning('接口请求失败, 进行第%d次尝试: %s' % (times + 1, e))
            if self._hasProxy and self.hasProxy().get('proxytype') == 'url': self.openProxy(self.session, self.hasProxy().get('proxystr'))
            times += 1
            return self.requestByType(type, times, *args, **kwargs)

    def requests_common(self, type, *args, **kwargs):
        res = self.requestByType(type, 0, *args, **kwargs)
        if not res.ok: logger.error(f'请求结果异常{res.status_code}：{res.text or "无内容"}')
        return res
    
    def get(self, *args, **kwargs):
        """
        发起get请求并对返回结果调用json方法，调用方式与requests.get一致

        **Parameters:**

        * **timeout_time:** `int` -- 超时尝试次数，默认为3次
        """
        return self.requests_common('get', *args, **kwargs)

    def post(self, *args, **kwargs):
        """
        发起post请求并对返回结果调用json方法，调用方式与requests.post一致

        **Parameters:**

        * **timeout_time:** `int` -- 超时尝试次数，默认为3次
        """
        return self.requests_common('post', *args, **kwargs)

    def requests_json_common(self, type, *args, **kwargs):
        ans = self.requestByType(type, 0, *args, **kwargs)
        try:
            data = ans.json()
            if hasattr(data, 'update'):
                data.update({ 'text': ans.text })
                return data
            return {
                    'data': data,
                    'text': ans.text,
                    }
        except Exception as e:
            logger.error('请求结果json解析失败: %s' % ans.text)
            if self.jsonErrorMessage: raise getvar(sdvar_exception)(self.jsonErrorNumber, self.jsonErrorMessage)
            return { 'sunday_error': True, 'msg': '返回结果非JSON', 'url': ans.request.url, 'text': ans.text, 'type': type }

    # get请求，返回dict对象
    def get_json(self, *args, **kwargs):
        """
        发起get请求并对返回结果调用json方法，调用方式与requests.get一致，由于对json方法做了安全处理建议在预期返回json数据的接口使用

        **Parameters:**

        * **timeout_time:** `int` -- 超时尝试次数，默认为3次
        """
        return self.requests_json_common('get', *args, **kwargs)

    # post请求，返回dict对象
    def post_json(self, *args, **kwargs):
        """
        发起post请求并对返回结果调用json方法，调用方式与requests.post一致，由于对json方法做了安全处理建议在预期返回json数据的接口使用

        **Parameters:**

        * **timeout_time:** `int` -- 超时尝试次数，默认为3次
        """
        return self.requests_json_common('post', *args, **kwargs)

    def getCookiesDict(self):
        """
        将cookie转化为字典并返回

        **Return:** `dict`
        """
        cookies = self.session.cookies
        if 'get_dict' in cookies:
            return cookies.get_dict()
        return requests.utils.dict_from_cookiejar(cookies)

    def setCookie(self, name, value, domain, rest={}, **argvs):
        """
        增加新的cookie, 入参参考cookielib.Cookie中的定义

        **Parameters:**

        * **name:** `str` -- cookie键名
        * **value:** `str` -- cookie键值
        * **domain:** `str` -- 主机名
        """
        cookies = self.session.cookies
        standard = { 'path': '/', 'domain': domain, 'version': 0 }
        standard.update(argvs)
        cookie = CookieJar()._cookie_from_cookie_tuple((name, value, standard, rest), None)
        cookies.set_cookie(cookie)



# coding: utf-8
import os
import sys
import re
import json
from pydash import pick as _pick
from sunday.core.globalvar import getvar
from sunday.core.globalKeyMaps import sdvar_logger

def clear():
    """清空屏幕"""
    os.system('clear')

def exit(text=''):
    """
    退出程序

    **Parameters:**

    * **text:** `str` -- 程序退出并打印退出提示文本
    """
    if text: getvar(sdvar_logger).critical(text)
    sys.exit(1)

def pascal_to_snake(text):
    """
    驼峰转蛇形

    **Usage:**

    ```
    >>> from sunday.core.common import pascal_to_snake
    >>> pascal_to_snake('HelloWorld')
    hello_world
    ```

    **Parameters:**

    * **text:** `str` -- 驼峰串
    """
    word = re.sub(r"(?P<key>[A-Z])", r"_\g<key>", text)
    return word.lower().strip('_')

def snake_to_pascal(text):
    """
    蛇形转驼峰

    **Usage:**

    ```
    >>> from sunday.core.common import snake_to_pascal
    >>> snake_to_pascal('hello_world')
    HelloWorld
    ```

    **Parameters:**

    * **text:** `str` -- 蛇形串
    """
    words = text.split('_')
    return ''.join(word.title() for word in words)

def parseJson(jsonPath, defaultValue={}, keywords=[], logger=None):
    """
    传入json文件，返回解析后的json数据

    **Parameters:**

    * **jsonPath:** `str` -- 文件路径
    * **defaultValue:** `list | dict` -- 文件读取失败时返回的默认数据
    * **keywords:** `list` -- 需要的键集合
    * **logger:** `str` -- 日志对象
    """
    if not logger: logger = getvar(sdvar_logger)
    if not os.path.exists(jsonPath): return defaultValue
    with open(jsonPath) as f:
        try:
            content = json.load(f)
            if type(defaultValue) == dict:
                return _pick(content, keywords)
            elif type(defaultValue) == list:
                return [_pick(cont, keywords) for cont in content]
        except Exception as e:
            logger.error('读取文件失败(%s): %s' % (jsonPath, e))
        return defaultValue



# coding: utf-8
import os
import types
from datetime import datetime

def mergeObj(ori={}, *objs):
    '''合并字典'''
    newObj = ori.copy()
    for obj in objs:
        newObj.update(obj)
    return newObj


def get_format_str(widths):
    str_width = sum(widths) + (len(widths) - 1) * 3 + 4
    ans = '|'
    for width in widths:
        ans += ' %-{}s'.format(width)
    return ans, str_width

def get_width(ans):
    width = 0
    times = 0
    if type(ans) in [bytes, str]:
        for s in ans:
            if ord(s) > 127:
                times += 1
        width = len(ans) + times
    return width, times

def parseJsonp(text, defvalue={}):
    """
    解析jsonp数据返回dict数据

    **Parameters:**

    * **text:** `str` -- jsonp数据字符串, 格式如：`jsonp_12345({})`
    """
    import json5
    try:
        start = text.find('(') + 1
        end = text.rfind(')')
        return json5.loads(text[start:end])
    except Exception as e:
        defvalue['sunday_msg'] = str(e)
        defvalue['sunday_ok'] = False
        return defvalue

def currentTimestamp():
    """
    返回当前时间戳, 单位为毫秒
    """
    return int(datetime.timestamp(datetime.now()) * 1000)



# coding: utf-8
# sunday 内部使用方法
import os
import sunday.core.paths as paths
from sunday.core.cmdexec import cmdexec
from sunday.core.globalvar import getvar
from sunday.core.globalKeyMaps import sdvar_logger
from sunday.core.common import parseJson
import sunday.core.paths as paths

__all__ = ['grenLoginAndToolsInit', 'printAllPlugin']

def grenLoginAndToolsInit():
    """生成插件的init导出文件"""
    logger = getvar(sdvar_logger)
    def grenInitFile(pathname):
        # 修改登录模块的__file__文件导出全部的登录模块
        filename = os.path.join(pathname, '__init__.py')
        filelist = list(filter(lambda n: n not in ['__init__.py', '__init__.pyc', '__pycache__'], os.listdir(pathname)))
        with open(filename, 'w') as f:
            f.write('\n'.join(['from . import %s' % i for i in filelist ]))
        logger.debug('文件内容:\n'.join([filename, cmdexec('cat %s' % filename)[1]]))
    grenInitFile(paths.sundayLoginCwd)
    grenInitFile(paths.sundayToolsCwd)

def printAllPlugin():
    # 打印所有插件信息
    logger = getvar(sdvar_logger)
    plugins = parseJson(paths.moduleLockCwd, [], ['origin', 'target', 'type', 'name'])
    pluginsArr = list(filter(lambda item: os.path.islink(item['target']), plugins))
    pluginsStr = '、'.join([item['type'] + '/' + item['name'] for item in pluginsArr])
    cmdStr = '、'.join(os.listdir(paths.binCwd))
    logger.info('已安装插件: %s' % pluginsStr)
    logger.info('可执行命令: %s' % cmdStr)



from sunday.core.globalvar import getvar, setvar
from sunday.core.globalKeyMaps import sdvar_exception
from pydash.objects import pick
from pydash.arrays import union

class SundayError(Exception): pass

def getException(errorMap={}, allowAttr=[]):
    """
    生成报错捕获对象，继承于CustomError > SundayError，默认存在属性code、message

    **Usage:**

    ```
    >>> from sunday.core.getException import getException
    >>> MyError = getException({ 10000: '错误提示文本' })
    >>> raise MyError(10000)
    ```

    **Parameters:**

    * **errorMap:** `dict` -- 错误映射字典，一般就是数字或字符键对应报错说明
    * **allowAttr:** `list` -- 允许的额外属性键列表

    **Return:** `NewError`
    """
    class CustomError(SundayError):
        def __init__(self, code=10001, message=None, other='', errorMap=errorMap, **kwargs):
            self.code = code
            tip = message or errorMap.get(code, '未知code: %s' % code)
            self.message = tip + ('(%s)' % other if other else '')
            kwargs = pick(kwargs, allowAttr) if type(allowAttr) == list and len(allowAttr) > 0 else kwargs
            for key in union([i for i in kwargs.keys()], allowAttr):
                setattr(self, key, kwargs.get(key) or '')

        def __str__(self):
            return repr('%d %s' % (self.code, self.message))

        def __iter__(self):
            iters = dict((x,y) for x, y in self.__dict__.items() if x[:2] != '__')
            iters.update(self.__dict__)
            for x, y in iters.items():
                yield x, y

    return CustomError

if not getvar(sdvar_exception):
    setvar(sdvar_exception, getException())



# coding: utf-8

if globals().get('_global_var') is None:
    global _global_var
    _global_var = {}

def getvar(name):
    """
    根据内部全局常量名返回值

    **Parameters:**

    * **name:** `str` -- 全局常量键名
    """
    return _global_var.get(name)

def setvar(name, value):
    """
    设置内部全局常量值

    **Parameters:**

    * **name:** `str` -- 全局常量键名
    * **value:** `str` -- 全局常量值
    """
    _global_var[name] = value
